@@ -56,7 +56,7 @@
 #'         \le} \code{resolution}. If \code{davies.baddeley} is supplied as a vector of
 #'         length 3, then the values \code{[1], [2], and [3]} correspond to the
 #'         parameters \eqn{\delta}, \eqn{\beta}, and \eqn{L_M=L_N} in Davies and
-#'         Baddeley (2017). If the argument is simply a single numeric value, it is
+#'         Baddeley (2018). If the argument is simply a single numeric value, it is
 #'         used for both \eqn{\delta} and \eqn{\beta}, with
 #'         \eqn{L_M=L_N=}\code{resolution} (i.e. no edge-correction surface
 #'         coarsening).
@@ -110,7 +110,7 @@
 #'   \code{\link[spatstat]{im}}) giving the pilot density to be used for
 #'   calculation of the variable bandwidths in adaptive estimation, \bold{or} a
 #'   \code{\link[spatstat]{ppp.object}} giving the data upon which to base a
-#'   fixed-bandwidth pilot estimate using \code{hp[1]}. If used, the pixel image
+#'   fixed-bandwidth pilot estimate using \code{hp}. If used, the pixel image
 #'   \emph{must} be defined over the same domain as the data given
 #' \code{resolution} or the supplied pre-set \code{xy} evaluation grid;
 #'   \bold{or} the planar point pattern data must be defined with respect to the
@@ -176,7 +176,7 @@
 #' --- a square root law, \emph{Annals of Statistics}, \bold{10}(4),
 #' 1217-1223.
 #' 
-#' Davies, T.M. and Baddeley A. (2017), Fast computation of
+#' Davies, T.M. and Baddeley A. (2018), Fast computation of
 #' spatially adaptive kernel estimates, \emph{Statistics and Computing}, [to appear].
 #' 
 #' Davies, T.M. and Hazelton, M.L. (2010), Adaptive kernel estimation of spatial relative
@@ -249,7 +249,7 @@ bivariate.density <- function(pp,h0,hp=NULL,adapt=FALSE,resolution=128,gamma.sca
 		if(is.null(hp)) hp <- h0
 		else hp <- checkit(hp,"'hp'")
 		
-		if(leaveoneout) return(bivden.LOO(pp,h0,hp,gamma.scale,trim,resolution,parallelise,weights))
+		if(leaveoneout) return(bivden.LOO(pp,h0,hp,(edge=="uniform"||edge=="diggle"),gamma.scale,trim,resolution,parallelise,weights,0)[[1]])
     
 		pd <- pilot.density
 		pilot.data <- pp
